from whoosh.qparser.dateparse import *


basedate = datetime(2010, 9, 20, 15, 16, 6, 454000)
english = English()


def assert_adatetime(at, **kwargs):
    assert at.__class__ is adatetime
    for key in adatetime.units:
        val = getattr(at, key)
        target = kwargs.get(key)
        assert val == target, "at.%s=%r not %r in %r" % (key, val, target, at)


def assert_timespan(ts, sargs, eargs):
    assert_adatetime(ts.start, **sargs)


def assert_unamb(ts, **kwargs):
    assert_unamb_span(ts, kwargs, kwargs)


def assert_unamb_span(ts, sargs, eargs):
    startdt = adatetime(**sargs).floor()
    enddt = adatetime(**eargs).ceil()
    assert ts.start == startdt, "start %s != %s" % (ts.start, startdt)
    assert ts.end == enddt, "end %s != %s" % (ts.end, enddt)


def assert_datespan(ts, startdate, enddate):
    assert ts.__class__ is timespan
    assert ts.start == startdate
    assert ts.end == enddate


#

def test_simple_dateparse(t=english.simple):
    assert_adatetime(t.date_from("2005", basedate), year=2005)
    assert_adatetime(t.date_from("200505", basedate), year=2005, month=5)
    assert_adatetime(t.date_from("20050510", basedate), year=2005, month=5,
                     day=10)
    assert_adatetime(t.date_from("2005051001", basedate),
                     year=2005, month=5, day=10, hour=1)
    assert_adatetime(t.date_from("200505100108", basedate),
                     year=2005, month=5, day=10, hour=1, minute=8)
    assert_adatetime(t.date_from("20050510010835", basedate),
                     year=2005, month=5, day=10, hour=1, minute=8, second=35)

    assert_adatetime(t.date_from("2005-05", basedate),
                     year=2005, month=5)
    assert_adatetime(t.date_from("2005 05 10", basedate),
                     year=2005, month=5, day=10)
    assert_adatetime(t.date_from("2005.05.10.01", basedate),
                     year=2005, month=5, day=10, hour=1)
    assert_adatetime(t.date_from("2005/05/10 01:08", basedate),
                     year=2005, month=5, day=10, hour=1, minute=8)
    assert_adatetime(t.date_from("2005.05.10  01:08:35", basedate),
                     year=2005, month=5, day=10, hour=1, minute=8, second=35)

    assert t.date_from("2005 02 31", basedate) is None
    assert t.date_from("2005-13-32", basedate) is None


def test_time(t=english.time):
    assert_adatetime(t.date_from("13:05", basedate), hour=13, minute=5)
    assert t.date_from("28:91", basedate) is None

    assert_adatetime(t.date_from("3pm", basedate), hour=15)
    assert_adatetime(t.date_from("3 pm", basedate), hour=15)
    assert_adatetime(t.date_from("10pm", basedate), hour=22)
    assert_adatetime(t.date_from("10 pm", basedate), hour=22)
    assert_adatetime(t.date_from("3am", basedate), hour=3)
    assert_adatetime(t.date_from("3:15 am", basedate), hour=3, minute=15)
    assert_adatetime(t.date_from("5:10pm", basedate), hour=17, minute=10)
    assert_adatetime(t.date_from("12:45am", basedate), hour=0, minute=45)
    assert_adatetime(t.date_from("12:45pm", basedate), hour=12, minute=45)
    assert_adatetime(t.date_from("5:45:05 pm", basedate),
                     hour=17, minute=45, second=5)

    assert_adatetime(t.date_from("noon", basedate),
                     hour=12, minute=0, second=0, microsecond=0)
    assert_adatetime(t.date_from("midnight", basedate),
                     hour=0, minute=0, second=0, microsecond=0)

    assert t.date_from("15 am", basedate) is None
    assert t.date_from("24:00", basedate) is None
    assert t.date_from("12:65", basedate) is None


def test_dmy(d=english.dmy):
    assert_adatetime(d.date_from("25 may 2011", basedate),
                     year=2011, month=5, day=25)
    assert_adatetime(d.date_from("may 2 2011", basedate),
                     year=2011, month=5, day=2)
    assert_adatetime(d.date_from("2011 25 may", basedate),
                     year=2011, month=5, day=25)
    assert_adatetime(d.date_from("2011 may 5", basedate),
                     year=2011, month=5, day=5)

    assert_adatetime(d.date_from("apr", basedate), month=4)
    assert_adatetime(d.date_from("september", basedate), month=9)
    assert_adatetime(d.date_from("2001", basedate), year=2001)
    assert_adatetime(d.date_from("july 2525", basedate), year=2525, month=7)
    assert_adatetime(d.date_from("nov 30", basedate), month=11, day=30)
    assert d.date_from("25 2525", basedate) is None

    assert_adatetime(d.date_from("25 may, 2011", basedate),
                     year=2011, month=5, day=25)
    assert_adatetime(d.date_from("may 2nd, 2011", basedate),
                     year=2011, month=5, day=2)
    assert_adatetime(d.date_from("2011, 25 may", basedate),
                     year=2011, month=5, day=25)
    assert_adatetime(d.date_from("2011, may 5th", basedate),
                     year=2011, month=5, day=5)

    assert_adatetime(d.date_from("today", basedate),
                     year=2010, month=9, day=20)
    assert_adatetime(d.date_from("tomorrow", basedate),
                     year=2010, month=9, day=21)
    assert_adatetime(d.date_from("yesterday", basedate),
                     year=2010, month=9, day=19)
    assert_adatetime(d.date_from("this month", basedate), year=2010, month=9)
    assert_adatetime(d.date_from("this year", basedate), year=2010)

    assert d.date_from("now", basedate) == basedate


def test_plustime(rt=english.plusdate):
    assert rt.date_from("+1hr", basedate) == basedate + timedelta(hours=1)
    assert rt.date_from("+5mins", basedate) == basedate + timedelta(minutes=5)
    assert rt.date_from("+20s", basedate) == basedate + timedelta(seconds=20)

    assert rt.date_from("- 2 h", basedate) == basedate + timedelta(hours=-2)
    assert rt.date_from("- 25 minutes", basedate) == basedate + timedelta(minutes=-25)
    assert rt.date_from("-400 secs", basedate) == basedate + timedelta(seconds=-400)

    assert rt.date_from("+1hr 5m", basedate) == basedate + timedelta(hours=1, minutes=5)
    assert rt.date_from("-8hr 12m", basedate) == basedate + timedelta(hours=-8, minutes=-12)
    assert rt.date_from("+1hr 5s", basedate) == basedate + timedelta(hours=1, seconds=5)
    assert rt.date_from("+1hr 12m 5s", basedate) == basedate + timedelta(hours=1, minutes=12, seconds=5)
    assert rt.date_from("-1hr 5s", basedate) == basedate + timedelta(hours=-1, seconds=-5)
    assert rt.date_from("-1hr 12m 5s", basedate) == basedate + timedelta(hours=-1, minutes=-12, seconds=-5)


def test_relative_days():
    # "next monday" on monday
    assert relative_days(0, 0, 1) == 7
    # "last monday" on monday
    assert relative_days(0, 0, -1) == -7
    # "next tuesday" on wednesday
    assert relative_days(2, 1, 1) == 6
    # "last tuesday" on wednesay
    assert relative_days(2, 1, -1) == -1
    # "last monday" on sunday
    assert relative_days(6, 0, -1) == -6
    # "next monday" on sunday
    assert relative_days(6, 0, 1) == 1
    # "next wednesday" on tuesday
    assert relative_days(1, 2, 1) == 1
    # "last wednesday" on tuesday
    assert relative_days(1, 2, -1) == -6
    # "last wednesday" on thursday
    assert relative_days(3, 2, -1) == -1
    # "next wednesday" on thursday
    assert relative_days(3, 2, 1) == 6
    # "last wednesday" on tuesday
    assert relative_days(1, 2, -1) == -6
    # "next wednesday" on tuesday
    assert relative_days(1, 2, 1) == 1


def test_dayname(p=english.dayname):
    assert_adatetime(p.date_from("next tuesday", basedate),
                     year=2010, month=9, day=21)
    assert_adatetime(p.date_from("last tuesday", basedate),
                     year=2010, month=9, day=14)
    assert_adatetime(p.date_from("next sunday", basedate),
                     year=2010, month=9, day=26)
    assert_adatetime(p.date_from("last sun", basedate),
                     year=2010, month=9, day=19)
    assert_adatetime(p.date_from("next th", basedate),
                     year=2010, month=9, day=23)


def test_reldate(p=english.plusdate):
    assert p.date_from("+1y", basedate) == basedate + relativedelta(years=1)
    assert p.date_from("+2mo", basedate) == basedate + relativedelta(months=2)
    assert p.date_from("+3w", basedate) == basedate + relativedelta(weeks=3)
    assert p.date_from("+5d", basedate) == basedate + relativedelta(days=5)
    assert p.date_from("+5days", basedate) == basedate + relativedelta(days=5)

    assert p.date_from("-6yr", basedate) == basedate + relativedelta(years=-6)
    assert p.date_from("- 7 mons", basedate) == basedate + relativedelta(months=-7)
    assert p.date_from("-8 wks", basedate) == basedate + relativedelta(weeks=-8)
    assert p.date_from("- 9 dy", basedate) == basedate + relativedelta(days=-9)

    assert p.date_from("+1y 12mo 400d", basedate) == basedate + relativedelta(years=1, months=12, days=400)
    assert p.date_from("-7mo 8d", basedate) == basedate + relativedelta(months=-7, days=-8)
    assert p.date_from("+5wks 2d", basedate) == basedate + relativedelta(weeks=5, days=2)
    assert p.date_from("-1y 1w", basedate) == basedate + relativedelta(years=-1, weeks=-1)

    assert p.date_from("+1y 2d 5h 12s", basedate) == basedate + relativedelta(years=1, days=2, hours=5, seconds=12)


def test_bundle_subs(p=english.bundle):
    test_time(p)
    test_dmy(p)
    test_plustime(p)
    test_dayname(p)
    test_reldate(p)


def test_bundle(p=english.bundle):
    assert_adatetime(p.date_from("mar 29 1972 2:45am", basedate),
                     year=1972, month=3, day=29, hour=2, minute=45)
    assert_adatetime(p.date_from("16:10:45 14 February 2005", basedate),
                     year=2005, month=2, day=14, hour=16, minute=10, second=45)
    assert_adatetime(p.date_from("1985 sept 12 12:01", basedate),
                     year=1985, month=9, day=12, hour=12, minute=1)
    assert_adatetime(p.date_from("5pm 21st oct 2005", basedate),
                     year=2005, month=10, day=21, hour=17)
    assert_adatetime(p.date_from("5:59:59pm next thur", basedate),
                     year=2010, month=9, day=23, hour=17, minute=59, second=59)


def test_ranges(p=english.torange):
    assert_timespan(p.date_from("last tuesday to next tuesday", basedate),
                         dict(year=2010, month=9, day=14),
                         dict(year=2010, month=9, day=21))
    assert_timespan(p.date_from("last monday to dec 25", basedate),
                         dict(year=2010, month=9, day=13),
                         dict(year=None, month=12, day=25))
    assert_timespan(p.date_from("oct 25 to feb 14", basedate),
                         dict(year=None, month=10, day=25),
                         dict(year=None, month=2, day=14))
    assert_timespan(p.date_from("3am oct 12 to 5pm", basedate),
                         dict(year=None, month=10, day=12, hour=3),
                         dict(year=None, month=None, day=None, hour=17))
    assert_timespan(p.date_from("3am feb 12 to 5pm today", basedate),
                         dict(year=None, month=2, day=12, hour=3),
                         dict(year=2010, month=9, day=20, hour=17))
    assert_timespan(p.date_from("feb to oct", basedate),
                         dict(year=None, month=2),
                         dict(year=None, month=10))
    assert_timespan(p.date_from("oct 25 2005 11am to 5pm tomorrow", basedate),
                         dict(year=2005, month=10, day=25, hour=11),
                         dict(year=2010, month=9, day=21, hour=17))
    assert_timespan(p.date_from("oct 5 2005 to november 20", basedate),
                         dict(year=2005, month=10, day=5),
                         dict(year=None, month=11, day=20))
    assert_timespan(p.date_from("2007 to 2010", basedate),
                         dict(year=2007, month=None, day=None),
                         dict(year=2010, month=None, day=None))
    assert_timespan(p.date_from("2007 to oct 12", basedate),
                         dict(year=2007, month=None, day=None),
                         dict(year=None, month=10, day=12))

    assert_datespan(p.date_from("-2d to +1w", basedate),
                         basedate + relativedelta(days=-2),
                         basedate + relativedelta(weeks=1))


def test_all():
    p = english.all
    test_bundle_subs(p)
    test_bundle(p)
    test_ranges(p)


def test_final_dates(p=english):
    assert_unamb(p.date_from("5:10pm", basedate),
                      year=2010, month=9, day=20, hour=17, minute=10)

    assert p.date_from("may 32 2005", basedate) is None
    assert p.date_from("2005 may 32", basedate) is None
    assert p.date_from("2005-13-32", basedate) is None


def test_final_ranges(p=english):
    assert_unamb_span(p.date_from("feb to nov", basedate),
                           dict(year=2010, month=2),
                           dict(year=2010, month=11))

    # 2005 to 10 oct 2009 -> jan 1 2005 to oct 31 2009
    assert_unamb_span(p.date_from("2005 to 10 oct 2009", basedate),
                           dict(year=2005),
                           dict(year=2009, month=10, day=10))

    # jan 12 to oct 10 2009 -> jan 12 2009 to oct 10 2009
    assert_unamb_span(p.date_from("jan 12 to oct 10 2009", basedate),
                           dict(year=2009, month=1, day=12),
                           dict(year=2009, month=10, day=10))

    # jan to oct 2009 -> jan 1 2009 to oct 31 2009
    assert_unamb_span(p.date_from("jan to oct 2009", basedate),
                           dict(year=2009, month=1),
                           dict(year=2009, month=10, day=31))

    # mar 2005 to oct -> mar 1 2005 to oct 31 basedate.year
    assert_unamb_span(p.date_from("mar 2005 to oct", basedate),
                           dict(year=2005, month=3),
                           dict(year=2010, month=10, day=31))

    # jan 10 to jan 25 -> jan 10 basedate.year to jan 25 basedate.year
    assert_unamb_span(p.date_from("jan 10 to jan 25", basedate),
                           dict(year=2010, month=1, day=10),
                           dict(year=2010, month=1, day=25))

    # jan 2005 to feb 2009 -> jan 1 2005 to feb 28 2009
    assert_unamb_span(p.date_from("jan 2005 to feb 2009", basedate),
                           dict(year=2005, month=1),
                           dict(year=2009, month=2))

    # jan 5000 to mar -> jan 1 5000 to mar 5000
    assert_unamb_span(p.date_from("jan 5000 to mar", basedate),
                           dict(year=5000, month=1),
                           dict(year=5000, month=3))

    # jun 5000 to jan -> jun 1 5000 to jan 31 5001
    assert_unamb_span(p.date_from("jun 5000 to jan", basedate),
                           dict(year=5000, month=6),
                           dict(year=5001, month=1))

    # oct 2010 to feb -> oct 1 2010 to feb 28 2011
    assert_unamb_span(p.date_from("oct 2010 to feb", basedate),
                           dict(year=2010, month=10),
                           dict(year=2011, month=2))

    assert_unamb_span(p.date_from("5pm to 3am", basedate),
                           dict(year=2010, month=9, day=20, hour=17),
                           dict(year=2010, month=9, day=21, hour=3))

    assert_unamb_span(p.date_from("5am to 3 am tomorrow", basedate),
                           dict(year=2010, month=9, day=20, hour=5),
                           dict(year=2010, month=9, day=21, hour=3))

    assert_unamb_span(p.date_from("3am to 5 pm tomorrow", basedate),
                           dict(year=2010, month=9, day=21, hour=3),
                           dict(year=2010, month=9, day=21, hour=17))

    assert_unamb_span(p.date_from("-2hrs to +20min", basedate),
                           dict(year=2010, month=9, day=20, hour=13, minute=16,
                                second=6, microsecond=454000),
                           dict(year=2010, month=9, day=20, hour=15, minute=36,
                                second=6, microsecond=454000))

    # Swap
    assert_unamb_span(p.date_from("oct 25 2009 to feb 14 2008", basedate),
                           dict(year=2008, month=2, day=14),
                           dict(year=2009, month=10, day=25))

    assert_unamb_span(p.date_from("oct 25 5000 to tomorrow", basedate),
                           dict(year=2010, month=9, day=21),
                           dict(year=5000, month=10, day=25))



# Copyright 2010 Matt Chaput. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    1. Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#
#    2. Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY MATT CHAPUT ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL MATT CHAPUT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# The views and conclusions contained in the software and documentation are
# those of the authors and should not be interpreted as representing official
# policies, either expressed or implied, of Matt Chaput.

"""
This module contains common utility objects/functions for the other query
parser modules.
"""

import sys

from whoosh.compat import string_type


class QueryParserError(Exception):
    def __init__(self, cause, msg=None):
        super(QueryParserError, self).__init__(str(cause))
        self.cause = cause


def get_single_text(field, text, **kwargs):
    """Returns the first token from an analyzer's output.
    """

    for t in field.process_text(text, mode="query", **kwargs):
        return t


def attach(q, stxnode):
    if q:
        q.startchar = stxnode.startchar
        q.endchar = stxnode.endchar
    return q


def print_debug(level, msg, out=sys.stderr):
    if level:
        out.write("%s%s\n" % (" " * (level - 1), msg))



# Copyright 2009 Matt Chaput. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    1. Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#
#    2. Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY MATT CHAPUT ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL MATT CHAPUT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# The views and conclusions contained in the software and documentation are
# those of the authors and should not be interpreted as representing official
# policies, either expressed or implied, of Matt Chaput.

"""
This module implements an FST/FSA writer and reader. An FST (Finite State
Transducer) stores a directed acyclic graph with values associated with the
leaves. Common elements of the values are pushed inside the tree. An FST that
does not store values is a regular FSA.

The format of the leaf values is pluggable using subclasses of the Values
class.

Whoosh uses these structures to store a directed acyclic word graph (DAWG) for
use in (at least) spell checking.
"""


import sys, copy
from array import array
from hashlib import sha1  # @UnresolvedImport

from whoosh.compat import b, u, BytesIO
from whoosh.compat import xrange, iteritems, iterkeys, izip, array_tobytes
from whoosh.compat import bytes_type, text_type
from whoosh.filedb.structfile import StructFile
from whoosh.system import _INT_SIZE
from whoosh.system import pack_byte, pack_int, pack_uint, pack_long
from whoosh.system import emptybytes
from whoosh.util.text import utf8encode, utf8decode
from whoosh.util.varints import varint


class FileVersionError(Exception):
    pass


class InactiveCursor(Exception):
    pass


ARC_LAST = 1
ARC_ACCEPT = 2
ARC_STOP = 4
ARC_HAS_VAL = 8
ARC_HAS_ACCEPT_VAL = 16
MULTIBYTE_LABEL = 32


# FST Value types

class Values(object):
    """Base for classes the describe how to encode and decode FST values.
    """

    @staticmethod
    def is_valid(v):
        """Returns True if v is a valid object that can be stored by this
        class.
        """

        raise NotImplementedError

    @staticmethod
    def common(v1, v2):
        """Returns the "common" part of the two values, for whatever "common"
        means for this class. For example, a string implementation would return
        the common shared prefix, for an int implementation it would return
        the minimum of the two numbers.

        If there is no common part, this method should return None.
        """

        raise NotImplementedError

    @staticmethod
    def add(prefix, v):
        """Adds the given prefix (the result of a call to common()) to the
        given value.
        """

        raise NotImplementedError

    @staticmethod
    def subtract(v, prefix):
        """Subtracts the "common" part (the prefix) from the given value.
        """

        raise NotImplementedError

    @staticmethod
    def write(dbfile, v):
        """Writes value v to a file.
        """

        raise NotImplementedError

    @staticmethod
    def read(dbfile):
        """Reads a value from the given file.
        """

        raise NotImplementedError

    @classmethod
    def skip(cls, dbfile):
        """Skips over a value in the given file.
        """

        cls.read(dbfile)

    @staticmethod
    def to_bytes(v):
        """Returns a str (Python 2.x) or bytes (Python 3) representation of
        the given value. This is used for calculating node digests, so it
        should be unique but fast to calculate, and does not have to be
        parseable.
        """

        raise NotImplementedError

    @staticmethod
    def merge(v1, v2):
        raise NotImplementedError


class IntValues(Values):
    """Stores integer values in an FST.
    """

    @staticmethod
    def is_valid(v):
        return isinstance(v, int) and v >= 0

    @staticmethod
    def common(v1, v2):
        if v1 is None or v2 is None:
            return None
        if v1 == v2:
            return v1
        return min(v1, v2)

    @staticmethod
    def add(base, v):
        if base is None:
            return v
        if v is None:
            return base
        return base + v

    @staticmethod
    def subtract(v, base):
        if v is None:
            return None
        if base is None:
            return v
        return v - base

    @staticmethod
    def write(dbfile, v):
        dbfile.write_uint(v)

    @staticmethod
    def read(dbfile):
        return dbfile.read_uint()

    @staticmethod
    def skip(dbfile):
        dbfile.seek(_INT_SIZE, 1)

    @staticmethod
    def to_bytes(v):
        return pack_int(v)


class SequenceValues(Values):
    """Abstract base class for value types that store sequences.
    """

    @staticmethod
    def is_valid(v):
        return isinstance(self, (list, tuple))

    @staticmethod
    def common(v1, v2):
        if v1 is None or v2 is None:
            return None

        i = 0
        while i < len(v1) and i < len(v2):
            if v1[i] != v2[i]:
                break
            i += 1

        if i == 0:
            return None
        if i == len(v1):
            return v1
        if i == len(v2):
            return v2
        return v1[:i]

    @staticmethod
    def add(prefix, v):
        if prefix is None:
            return v
        if v is None:
            return prefix
        return prefix + v

    @staticmethod
    def subtract(v, prefix):
        if prefix is None:
            return v
        if v is None:
            return None
        if len(v) == len(prefix):
            return None
        if len(v) < len(prefix) or len(prefix) == 0:
            raise ValueError((v, prefix))
        return v[len(prefix):]

    @staticmethod
    def write(dbfile, v):
        dbfile.write_pickle(v)

    @staticmethod
    def read(dbfile):
        return dbfile.read_pickle()


class BytesValues(SequenceValues):
    """Stores bytes objects (str in Python 2.x) in an FST.
    """

    @staticmethod
    def is_valid(v):
        return isinstance(v, bytes_type)

    @staticmethod
    def write(dbfile, v):
        dbfile.write_int(len(v))
        dbfile.write(v)

    @staticmethod
    def read(dbfile):
        length = dbfile.read_int()
        return dbfile.read(length)

    @staticmethod
    def skip(dbfile):
        length = dbfile.read_int()
        dbfile.seek(length, 1)

    @staticmethod
    def to_bytes(v):
        return v


class ArrayValues(SequenceValues):
    """Stores array.array objects in an FST.
    """

    def __init__(self, typecode):
        self.typecode = typecode
        self.itemsize = array(self.typecode).itemsize

    def is_valid(self, v):
        return isinstance(v, array) and v.typecode == self.typecode

    @staticmethod
    def write(dbfile, v):
        dbfile.write(b(v.typecode))
        dbfile.write_int(len(v))
        dbfile.write_array(v)

    def read(self, dbfile):
        typecode = u(dbfile.read(1))
        length = dbfile.read_int()
        return dbfile.read_array(self.typecode, length)

    def skip(self, dbfile):
        length = dbfile.read_int()
        dbfile.seek(length * self.itemsize, 1)

    @staticmethod
    def to_bytes(v):
        return array_tobytes(v)


class IntListValues(SequenceValues):
    """Stores lists of positive, increasing integers (that is, lists of
    integers where each number is >= 0 and each number is greater than or equal
    to the number that precedes it) in an FST.
    """

    @staticmethod
    def is_valid(v):
        if isinstance(v, (list, tuple)):
            if len(v) < 2:
                return True
            for i in xrange(1, len(v)):
                if not isinstance(v[i], int) or v[i] < v[i - 1]:
                    return False
            return True
        return False

    @staticmethod
    def write(dbfile, v):
        base = 0
        dbfile.write_varint(len(v))
        for x in v:
            delta = x - base
            assert delta >= 0
            dbfile.write_varint(delta)
            base = x

    @staticmethod
    def read(dbfile):
        length = dbfile.read_varint()
        result = []
        if length > 0:
            base = 0
            for _ in xrange(length):
                base += dbfile.read_varint()
                result.append(base)
        return result

    @staticmethod
    def to_bytes(v):
        return b(repr(v))


# Node-like interface wrappers

class Node(object):
    """A slow but easier-to-use wrapper for FSA/DAWGs. Translates the low-level
    arc-based interface of GraphReader into Node objects with methods to follow
    edges.
    """

    def __init__(self, owner, address, accept=False):
        self.owner = owner
        self.address = address
        self._edges = None
        self.accept = accept

    def __iter__(self):
        if not self._edges:
            self._load()
        return iterkeys(self._edges)

    def __contains__(self, key):
        if self._edges is None:
            self._load()
        return key in self._edges

    def _load(self):
        owner = self.owner
        if self.address is None:
            d = {}
        else:
            d = dict((arc.label, Node(owner, arc.target, arc.accept))
                     for arc in self.owner.iter_arcs(self.address))
        self._edges = d

    def keys(self):
        if self._edges is None:
            self._load()
        return self._edges.keys()

    def all_edges(self):
        if self._edges is None:
            self._load()
        return self._edges

    def edge(self, key):
        if self._edges is None:
            self._load()
        return self._edges[key]

    def flatten(self, sofar=emptybytes):
        if self.accept:
            yield sofar
        for key in sorted(self):
            node = self.edge(key)
            for result in node.flatten(sofar + key):
                yield result

    def flatten_strings(self):
        return (utf8decode(k)[0] for k in self.flatten())


class ComboNode(Node):
    """Base class for nodes that blend the nodes of two different graphs.

    Concrete subclasses need to implement the ``edge()`` method and possibly
    override the ``accept`` property.
    """

    def __init__(self, a, b):
        self.a = a
        self.b = b

    def __repr__(self):
        return "<%s %r %r>" % (self.__class__.__name__, self.a, self.b)

    def __contains__(self, key):
        return key in self.a or key in self.b

    def __iter__(self):
        return iter(set(self.a) | set(self.b))

    @property
    def accept(self):
        return self.a.accept or self.b.accept


class UnionNode(ComboNode):
    """Makes two graphs appear to be the union of the two graphs.
    """

    def edge(self, key):
        a = self.a
        b = self.b
        if key in a and key in b:
            return UnionNode(a.edge(key), b.edge(key))
        elif key in a:
            return a.edge(key)
        else:
            return b.edge(key)


class IntersectionNode(ComboNode):
    """Makes two graphs appear to be the intersection of the two graphs.
    """

    def edge(self, key):
        a = self.a
        b = self.b
        if key in a and key in b:
            return IntersectionNode(a.edge(key), b.edge(key))


# Cursor

class BaseCursor(object):
    """Base class for a cursor-type object for navigating an FST/word graph,
    represented by a :class:`GraphReader` object.

    >>> cur = GraphReader(dawgfile).cursor()
    >>> for key in cur.follow():
    ...   print(repr(key))

    The cursor "rests" on arcs in the FSA/FST graph, rather than nodes.
    """

    def is_active(self):
        """Returns True if this cursor is still active, that is it has not
        read past the last arc in the graph.
        """

        raise NotImplementedError

    def label(self):
        """Returns the label bytes of the current arc.
        """

        raise NotImplementedError

    def prefix(self):
        """Returns a sequence of the label bytes for the path from the root
        to the current arc.
        """

        raise NotImplementedError

    def prefix_bytes(self):
        """Returns the label bytes for the path from the root to the current
        arc as a single joined bytes object.
        """

        return emptybytes.join(self.prefix())

    def prefix_string(self):
        """Returns the labels of the path from the root to the current arc as
        a decoded unicode string.
        """

        return utf8decode(self.prefix_bytes())[0]

    def peek_key(self):
        """Returns a sequence of label bytes representing the next closest
        key in the graph.
        """

        for label in self.prefix():
            yield label
        c = self.copy()
        while not c.stopped():
            c.follow()
            yield c.label()

    def peek_key_bytes(self):
        """Returns the next closest key in the graph as a single bytes object.
        """

        return emptybytes.join(self.peek_key())

    def peek_key_string(self):
        """Returns the next closest key in the graph as a decoded unicode
        string.
        """

        return utf8decode(self.peek_key_bytes())[0]

    def stopped(self):
        """Returns True if the current arc leads to a stop state.
        """

        raise NotImplementedError

    def value(self):
        """Returns the value at the current arc, if reading an FST.
        """

        raise NotImplementedError

    def accept(self):
        """Returns True if the current arc leads to an accept state (the end
        of a valid key).
        """

        raise NotImplementedError

    def at_last_arc(self):
        """Returns True if the current arc is the last outgoing arc from the
        previous node.
        """

        raise NotImplementedError

    def next_arc(self):
        """Moves to the next outgoing arc from the previous node.
        """

        raise NotImplementedError

    def follow(self):
        """Follows the current arc.
        """

        raise NotImplementedError

    def switch_to(self, label):
        """Switch to the sibling arc with the given label bytes.
        """

        _label = self.label
        _at_last_arc = self.at_last_arc
        _next_arc = self.next_arc

        while True:
            thislabel = _label()
            if thislabel == label:
                return True
            if thislabel > label or _at_last_arc():
                return False
            _next_arc()

    def skip_to(self, key):
        """Moves the cursor to the path represented by the given key bytes.
        """

        _accept = self.accept
        _prefix = self.prefix
        _next_arc = self.next_arc

        keylist = list(key)
        while True:
            if _accept():
                thiskey = list(_prefix())
                if keylist == thiskey:
                    return True
                elif keylist > thiskey:
                    return False
            _next_arc()

    def flatten(self):
        """Yields the keys in the graph, starting at the current position.
        """

        _is_active = self.is_active
        _accept = self.accept
        _stopped = self.stopped
        _follow = self.follow
        _next_arc = self.next_arc
        _prefix_bytes = self.prefix_bytes

        if not _is_active():
            raise InactiveCursor
        while _is_active():
            if _accept():
                yield _prefix_bytes()
            if not _stopped():
                _follow()
                continue
            _next_arc()

    def flatten_v(self):
        """Yields (key, value) tuples in an FST, starting at the current
        position.
        """

        for key in self.flatten():
            yield key, self.value()

    def flatten_strings(self):
        return (utf8decode(k)[0] for k in self.flatten())

    def find_path(self, path):
        """Follows the labels in the given path, starting at the current
        position.
        """

        path = to_labels(path)
        _switch_to = self.switch_to
        _follow = self.follow
        _stopped = self.stopped

        first = True
        for i, label in enumerate(path):
            if not first:
                _follow()
            if not _switch_to(label):
                return False
            if _stopped():
                if i < len(path) - 1:
                    return False
            first = False
        return True


class Cursor(BaseCursor):
    def __init__(self, graph, root=None, stack=None):
        self.graph = graph
        self.vtype = graph.vtype
        self.root = root if root is not None else graph.default_root()
        if stack:
            self.stack = stack
        else:
            self.reset()

    def _current_attr(self, name):
        stack = self.stack
        if not stack:
            raise InactiveCursor
        return getattr(stack[-1], name)

    def is_active(self):
        return bool(self.stack)

    def stopped(self):
        return self._current_attr("target") is None

    def accept(self):
        return self._current_attr("accept")

    def at_last_arc(self):
        return self._current_attr("lastarc")

    def label(self):
        return self._current_attr("label")

    def reset(self):
        self.stack = []
        self.sums = [None]
        self._push(self.graph.arc_at(self.root))

    def copy(self):
        return self.__class__(self.graph, self.root, copy.deepcopy(self.stack))

    def prefix(self):
        stack = self.stack
        if not stack:
            raise InactiveCursor
        return (arc.label for arc in stack)

    # Override: more efficient implementation using graph methods directly
    def peek_key(self):
        if not self.stack:
            raise InactiveCursor

        for label in self.prefix():
            yield label
        arc = copy.copy(self.stack[-1])
        graph = self.graph
        while not arc.accept and arc.target is not None:
            graph.arc_at(arc.target, arc)
            yield arc.label

    def value(self):
        stack = self.stack
        if not stack:
            raise InactiveCursor
        vtype = self.vtype
        if not vtype:
            raise Exception("No value type")

        v = self.sums[-1]
        current = stack[-1]
        if current.value:
            v = vtype.add(v, current.value)
        if current.accept and current.acceptval is not None:
            v = vtype.add(v, current.acceptval)
        return v

    def next_arc(self):
        stack = self.stack
        if not stack:
            raise InactiveCursor

        while stack and stack[-1].lastarc:
            self.pop()
        if stack:
            current = stack[-1]
            self.graph.arc_at(current.endpos, current)
            return current

    def follow(self):
        address = self._current_attr("target")
        if address is None:
            raise Exception("Can't follow a stop arc")
        self._push(self.graph.arc_at(address))
        return self

    # Override: more efficient implementation manipulating the stack
    def skip_to(self, key):
        key = to_labels(key)
        stack = self.stack
        if not stack:
            raise InactiveCursor

        _follow = self.follow
        _next_arc = self.next_arc

        i = self._pop_to_prefix(key)
        while stack and i < len(key):
            curlabel = stack[-1].label
            keylabel = key[i]
            if curlabel == keylabel:
                _follow()
                i += 1
            elif curlabel > keylabel:
                return
            else:
                _next_arc()

    # Override: more efficient implementation using find_arc
    def switch_to(self, label):
        stack = self.stack
        if not stack:
            raise InactiveCursor

        current = stack[-1]
        if label == current.label:
            return True
        else:
            arc = self.graph.find_arc(current.endpos, label, current)
            return arc

    def _push(self, arc):
        if self.vtype and self.stack:
            sums = self.sums
            sums.append(self.vtype.add(sums[-1], self.stack[-1].value))
        self.stack.append(arc)

    def pop(self):
        self.stack.pop()
        if self.vtype:
            self.sums.pop()

    def _pop_to_prefix(self, key):
        stack = self.stack
        if not stack:
            raise InactiveCursor

        i = 0
        maxpre = min(len(stack), len(key))
        while i < maxpre and key[i] == stack[i].label:
            i += 1
        if stack[i].label > key[i]:
            self.current = None
            return
        while len(stack) > i + 1:
            self.pop()
        self.next_arc()
        return i


class UncompiledNode(object):
    # Represents an "in-memory" node used by the GraphWriter before it is
    # written to disk.

    compiled = False

    def __init__(self, owner):
        self.owner = owner
        self._digest = None
        self.clear()

    def clear(self):
        self.arcs = []
        self.value = None
        self.accept = False
        self.inputcount = 0

    def __repr__(self):
        return "<%r>" % ([(a.label, a.value) for a in self.arcs],)

    def digest(self):
        if self._digest is None:
            d = sha1()
            vtype = self.owner.vtype
            for arc in self.arcs:
                d.update(arc.label)
                if arc.target:
                    d.update(pack_long(arc.target))
                else:
                    d.update(b("z"))
                if arc.value:
                    d.update(vtype.to_bytes(arc.value))
                if arc.accept:
                    d.update(b("T"))
            self._digest = d.digest()
        return self._digest

    def edges(self):
        return self.arcs

    def last_value(self, label):
        assert self.arcs[-1].label == label
        return self.arcs[-1].value

    def add_arc(self, label, target):
        self.arcs.append(Arc(label, target))

    def replace_last(self, label, target, accept, acceptval=None):
        arc = self.arcs[-1]
        assert arc.label == label, "%r != %r" % (arc.label, label)
        arc.target = target
        arc.accept = accept
        arc.acceptval = acceptval

    def delete_last(self, label, target):
        arc = self.arcs.pop()
        assert arc.label == label
        assert arc.target == target

    def set_last_value(self, label, value):
        arc = self.arcs[-1]
        assert arc.label == label, "%r->%r" % (arc.label, label)
        arc.value = value

    def prepend_value(self, prefix):
        add = self.owner.vtype.add
        for arc in self.arcs:
            arc.value = add(prefix, arc.value)
        if self.accept:
            self.value = add(prefix, self.value)


class Arc(object):
    """
    Represents a directed arc between two nodes in an FSA/FST graph.

    The ``lastarc`` attribute is True if this is the last outgoing arc from the
    previous node.
    """

    __slots__ = ("label", "target", "accept", "value", "lastarc", "acceptval",
                 "endpos")

    def __init__(self, label=None, target=None, value=None, accept=False,
                 acceptval=None, lastarc=None, endpos=None):
        """
        :param label: The label bytes for this arc. For a word graph, this will
            be a character.
        :param target: The address of the node at the endpoint of this arc.
        :param value: The inner FST value at the endpoint of this arc.
        :param accept: Whether the endpoint of this arc is an accept state
            (e.g. the end of a valid word).
        :param acceptval: If the endpoint of this arc is an accept state, the
            final FST value for that accepted state.
        """

        self.label = label
        self.target = target
        self.value = value
        self.accept = accept
        self.acceptval = acceptval
        self.lastarc = lastarc
        self.endpos = endpos

    def __repr__(self):
        return "<%r-%s %s%s>" % (self.label, self.target,
                                 "." if self.accept else "",
                                 (" %r" % self.value) if self.value else "")

    def __eq__(self, other):
        if (isinstance(other, self.__class__) and self.accept == other.accept
            and self.lastarc == other.lastarc and self.target == other.target
            and self.value == other.value and self.label == other.label):
            return True
        return False

    def copy(self):
        # This is faster than using the copy module
        return Arc(label=self.label, target=self.target, value=self.value,
                   accept=self.accept, acceptval=self.acceptval,
                   lastarc=self.lastarc, endpos=self.endpos)


# Graph writer

class GraphWriter(object):
    """Writes an FSA/FST graph to disk.

    Call ``insert(key)`` to insert keys into the graph. You must
    insert keys in sorted order. Call ``close()`` to finish the graph and close
    the file.

    >>> gw = GraphWriter(my_file)
    >>> gw.insert("alfa")
    >>> gw.insert("bravo")
    >>> gw.insert("charlie")
    >>> gw.close()

    The graph writer can write separate graphs for multiple fields. Use
    ``start_field(name)`` and ``finish_field()`` to separate fields.

    >>> gw = GraphWriter(my_file)
    >>> gw.start_field("content")
    >>> gw.insert("alfalfa")
    >>> gw.insert("apple")
    >>> gw.finish_field()
    >>> gw.start_field("title")
    >>> gw.insert("artichoke")
    >>> gw.finish_field()
    >>> gw.close()
    """

    version = 1

    def __init__(self, dbfile, vtype=None, merge=None):
        """
        :param dbfile: the file to write to.
        :param vtype: a :class:`Values` class to use for storing values. This
            is only necessary if you will be storing values for the keys.
        :param merge: a function that takes two values and returns a single
            value. This is called if you insert two identical keys with values.
        """

        self.dbfile = dbfile
        self.vtype = vtype
        self.merge = merge
        self.fieldroots = {}
        self.arc_count = 0
        self.node_count = 0
        self.fixed_count = 0

        dbfile.write(b("GRPH"))
        dbfile.write_int(self.version)
        dbfile.write_uint(0)

        self._infield = False

    def start_field(self, fieldname):
        """Starts a new graph for the given field.
        """

        if not fieldname:
            raise ValueError("Field name cannot be equivalent to False")
        if self._infield:
            self.finish_field()
        self.fieldname = fieldname
        self.seen = {}
        self.nodes = [UncompiledNode(self)]
        self.lastkey = ''
        self._inserted = False
        self._infield = True

    def finish_field(self):
        """Finishes the graph for the current field.
        """

        if not self._infield:
            raise Exception("Called finish_field before start_field")
        self._infield = False
        if self._inserted:
            self.fieldroots[self.fieldname] = self._finish()
        self.fieldname = None

    def close(self):
        """Finishes the current graph and closes the underlying file.
        """

        if self.fieldname is not None:
            self.finish_field()
        dbfile = self.dbfile
        here = dbfile.tell()
        dbfile.write_pickle(self.fieldroots)
        dbfile.flush()
        dbfile.seek(4 + _INT_SIZE)  # Seek past magic and version number
        dbfile.write_uint(here)
        dbfile.close()

    def insert(self, key, value=None):
        """Inserts the given key into the graph.

        :param key: a sequence of bytes objects, a bytes object, or a string.
        :param value: an optional value to encode in the graph along with the
            key. If the writer was not instantiated with a value type, passing
            a value here will raise an error.
        """

        if not self._infield:
            raise Exception("Inserted %r before starting a field" % key)
        self._inserted = True
        key = to_labels(key)  # Python 3 sucks

        vtype = self.vtype
        lastkey = self.lastkey
        nodes = self.nodes
        if len(key) < 1:
            raise KeyError("Can't store a null key %r" % (key,))
        if lastkey and lastkey > key:
            raise KeyError("Keys out of order %r..%r" % (lastkey, key))

        # Find the common prefix shared by this key and the previous one
        prefixlen = 0
        for i in xrange(min(len(lastkey), len(key))):
            if lastkey[i] != key[i]:
                break
            prefixlen += 1
        # Compile the nodes after the prefix, since they're not shared
        self._freeze_tail(prefixlen + 1)

        # Create new nodes for the parts of this key after the shared prefix
        for char in key[prefixlen:]:
            node = UncompiledNode(self)
            # Create an arc to this node on the previous node
            nodes[-1].add_arc(char, node)
            nodes.append(node)
        # Mark the last node as an accept state
        lastnode = nodes[-1]
        lastnode.accept = True

        if vtype:
            if value is not None and not vtype.is_valid(value):
                raise ValueError("%r is not valid for %s" % (value, vtype))

            # Push value commonalities through the tree
            common = None
            for i in xrange(1, prefixlen + 1):
                node = nodes[i]
                parent = nodes[i - 1]
                lastvalue = parent.last_value(key[i - 1])
                if lastvalue is not None:
                    common = vtype.common(value, lastvalue)
                    suffix = vtype.subtract(lastvalue, common)
                    parent.set_last_value(key[i - 1], common)
                    node.prepend_value(suffix)
                else:
                    common = suffix = None
                value = vtype.subtract(value, common)

            if key == lastkey:
                # If this key is a duplicate, merge its value with the value of
                # the previous (same) key
                lastnode.value = self.merge(lastnode.value, value)
            else:
                nodes[prefixlen].set_last_value(key[prefixlen], value)
        elif value:
            raise Exception("Value %r but no value type" % value)

        self.lastkey = key

    def _freeze_tail(self, prefixlen):
        nodes = self.nodes
        lastkey = self.lastkey
        downto = max(1, prefixlen)

        while len(nodes) > downto:
            node = nodes.pop()
            parent = nodes[-1]
            inlabel = lastkey[len(nodes) - 1]

            self._compile_targets(node)
            accept = node.accept or len(node.arcs) == 0
            address = self._compile_node(node)
            parent.replace_last(inlabel, address, accept, node.value)

    def _finish(self):
        nodes = self.nodes
        root = nodes[0]
        # Minimize nodes in the last word's suffix
        self._freeze_tail(0)
        # Compile remaining targets
        self._compile_targets(root)
        return self._compile_node(root)

    def _compile_targets(self, node):
        for arc in node.arcs:
            if isinstance(arc.target, UncompiledNode):
                n = arc.target
                if len(n.arcs) == 0:
                    arc.accept = n.accept = True
                arc.target = self._compile_node(n)

    def _compile_node(self, uncnode):
        seen = self.seen

        if len(uncnode.arcs) == 0:
            # Leaf node
            address = self._write_node(uncnode)
        else:
            d = uncnode.digest()
            address = seen.get(d)
            if address is None:
                address = self._write_node(uncnode)
                seen[d] = address
        return address

    def _write_node(self, uncnode):
        vtype = self.vtype
        dbfile = self.dbfile
        arcs = uncnode.arcs
        numarcs = len(arcs)

        if not numarcs:
            if uncnode.accept:
                return None
            else:
                # What does it mean for an arc to stop but not be accepted?
                raise Exception
        self.node_count += 1

        buf = StructFile(BytesIO())
        nodestart = dbfile.tell()
        #self.count += 1
        #self.arccount += numarcs

        fixedsize = -1
        arcstart = buf.tell()
        for i, arc in enumerate(arcs):
            self.arc_count += 1
            target = arc.target
            label = arc.label

            flags = 0
            if len(label) > 1:
                flags += MULTIBYTE_LABEL
            if i == numarcs - 1:
                flags += ARC_LAST
            if arc.accept:
                flags += ARC_ACCEPT
            if target is None:
                flags += ARC_STOP
            if arc.value is not None:
                flags += ARC_HAS_VAL
            if arc.acceptval is not None:
                flags += ARC_HAS_ACCEPT_VAL

            buf.write(pack_byte(flags))
            if len(label) > 1:
                buf.write(varint(len(label)))
            buf.write(label)
            if target is not None:
                buf.write(pack_uint(target))
            if arc.value is not None:
                vtype.write(buf, arc.value)
            if arc.acceptval is not None:
                vtype.write(buf, arc.acceptval)

            here = buf.tell()
            thissize = here - arcstart
            arcstart = here
            if fixedsize == -1:
                fixedsize = thissize
            elif fixedsize > 0 and thissize != fixedsize:
                fixedsize = 0

        if fixedsize > 0:
            # Write a fake arc containing the fixed size and number of arcs
            dbfile.write_byte(255)  # FIXED_SIZE
            dbfile.write_int(fixedsize)
            dbfile.write_int(numarcs)
            self.fixed_count += 1
        dbfile.write(buf.file.getvalue())

        return nodestart


# Graph reader

class BaseGraphReader(object):
    def cursor(self, rootname=None):
        return Cursor(self, self.root(rootname))

    def has_root(self, rootname):
        raise NotImplementedError

    def root(self, rootname=None):
        raise NotImplementedError

    # Low level methods

    def arc_at(self, address, arc):
        raise NotImplementedError

    def iter_arcs(self, address, arc=None):
        raise NotImplementedError

    def find_arc(self, address, label, arc=None):
        arc = arc or Arc()
        for arc in self.iter_arcs(address, arc):
            thislabel = arc.label
            if thislabel == label:
                return arc
            elif thislabel > label:
                return None

    # Convenience methods

    def list_arcs(self, address):
        return list(arc.copy() for arc in self.iter_arcs(address))

    def arc_dict(self, address):
        return dict((arc.label, arc.copy())
                    for arc in self.iter_arcs(address))

    def find_path(self, path, arc=None, address=None):
        path = to_labels(path)

        if arc:
            address = address if address is not None else arc.target
        else:
            arc = Arc()

        if address is None:
            address = self._root

        find_arc = self.find_arc
        for label in path:
            if address is None:
                return None
            if not find_arc(address, label, arc):
                return None
            address = arc.target
        return arc


class GraphReader(BaseGraphReader):
    def __init__(self, dbfile, rootname=None, vtype=None, filebase=0):
        self.dbfile = dbfile
        self.vtype = vtype
        self.filebase = filebase

        dbfile.seek(filebase)
        magic = dbfile.read(4)
        if magic != b("GRPH"):
            raise FileVersionError
        self.version = dbfile.read_int()
        dbfile.seek(dbfile.read_uint())
        self.roots = dbfile.read_pickle()

        self._root = None
        if rootname is None and len(self.roots) == 1:
            # If there's only one root, just use it. Have to wrap a list around
            # the keys() method here because of Python 3.
            rootname = list(self.roots.keys())[0]
        if rootname is not None:
            self._root = self.root(rootname)

    def close(self):
        self.dbfile.close()

    # Overrides

    def has_root(self, rootname):
        return rootname in self.roots

    def root(self, rootname=None):
        if rootname is None:
            return self._root
        else:
            return self.roots[rootname]

    def default_root(self):
        return self._root

    def arc_at(self, address, arc=None):
        arc = arc or Arc()
        self.dbfile.seek(address)
        return self._read_arc(arc)

    def iter_arcs(self, address, arc=None):
        arc = arc or Arc()
        _read_arc = self._read_arc

        self.dbfile.seek(address)
        while True:
            _read_arc(arc)
            yield arc
            if arc.lastarc:
                break

    def find_arc(self, address, label, arc=None):
        # Overrides the default scanning implementation

        arc = arc or Arc()
        dbfile = self.dbfile
        dbfile.seek(address)

        # If records are fixed size, we can do a binary search
        finfo = self._read_fixed_info()
        if finfo:
            size, count = finfo
            address = dbfile.tell()
            if count > 2:
                return self._binary_search(address, size, count, label, arc)

        # If records aren't fixed size, fall back to the parent's linear
        # search method
        return BaseGraphReader.find_arc(self, address, label, arc)

    # Implementations

    def _read_arc(self, toarc=None):
        toarc = toarc or Arc()
        dbfile = self.dbfile
        flags = dbfile.read_byte()
        if flags == 255:
            # This is a fake arc containing fixed size information; skip it
            # and read the next arc
            dbfile.seek(_INT_SIZE * 2, 1)
            flags = dbfile.read_byte()
        toarc.label = self._read_label(flags)
        return self._read_arc_data(flags, toarc)

    def _read_label(self, flags):
        dbfile = self.dbfile
        if flags & MULTIBYTE_LABEL:
            length = dbfile.read_varint()
        else:
            length = 1
        label = dbfile.read(length)
        return label

    def _read_fixed_info(self):
        dbfile = self.dbfile

        flags = dbfile.read_byte()
        if flags == 255:
            size = dbfile.read_int()
            count = dbfile.read_int()
            return (size, count)
        else:
            return None

    def _read_arc_data(self, flags, arc):
        dbfile = self.dbfile
        accept = arc.accept = bool(flags & ARC_ACCEPT)
        arc.lastarc = flags & ARC_LAST
        if flags & ARC_STOP:
            arc.target = None
        else:
            arc.target = dbfile.read_uint()
        if flags & ARC_HAS_VAL:
            arc.value = self.vtype.read(dbfile)
        else:
            arc.value = None
        if accept and flags & ARC_HAS_ACCEPT_VAL:
            arc.acceptval = self.vtype.read(dbfile)
        arc.endpos = dbfile.tell()
        return arc

    def _binary_search(self, address, size, count, label, arc):
        dbfile = self.dbfile
        _read_label = self._read_label

        lo = 0
        hi = count
        while lo < hi:
            mid = (lo + hi) // 2
            midaddr = address + mid * size
            dbfile.seek(midaddr)
            flags = dbfile.read_byte()
            midlabel = self._read_label(flags)
            if midlabel == label:
                arc.label = midlabel
                return self._read_arc_data(flags, arc)
            elif midlabel < label:
                lo = mid + 1
            else:
                hi = mid
        if lo == count:
            return None


def to_labels(key):
    """Takes a string and returns a list of bytestrings, suitable for use as
    a key or path in an FSA/FST graph.
    """

    # Convert to tuples of bytestrings (must be tuples so they can be hashed)
    keytype = type(key)

    # I hate the Python 3 bytes object so friggin much
    if keytype is tuple or keytype is list:
        if not all(isinstance(e, bytes_type) for e in key):
            raise TypeError("%r contains a non-bytestring" % key)
        if keytype is list:
            key = tuple(key)
    elif isinstance(key, bytes_type):
        key = tuple(key[i:i + 1] for i in xrange(len(key)))
    elif isinstance(key, text_type):
        key = tuple(utf8encode(key[i:i + 1])[0] for i in xrange(len(key)))
    else:
        raise TypeError("Don't know how to convert %r" % key)
    return key


# Within edit distance function

def within(graph, text, k=1, prefix=0, address=None):
    """Yields a series of keys in the given graph within ``k`` edit distance of
    ``text``. If ``prefix`` is greater than 0, all keys must match the first
    ``prefix`` characters of ``text``.
    """

    text = to_labels(text)
    if address is None:
        address = graph._root

    sofar = emptybytes
    accept = False
    if prefix:
        prefixchars = text[:prefix]
        arc = graph.find_path(prefixchars, address=address)
        if arc is None:
            return
        sofar = emptybytes.join(prefixchars)
        address = arc.target
        accept = arc.accept

    stack = [(address, k, prefix, sofar, accept)]
    seen = set()
    while stack:
        state = stack.pop()
        # Have we already tried this state?
        if state in seen:
            continue
        seen.add(state)

        address, k, i, sofar, accept = state
        # If we're at the end of the text (or deleting enough chars would get
        # us to the end and still within K), and we're in the accept state,
        # yield the current result
        if (len(text) - i <= k) and accept:
            yield utf8decode(sofar)[0]

        # If we're in the stop state, give up
        if address is None:
            continue

        # Exact match
        if i < len(text):
            arc = graph.find_arc(address, text[i])
            if arc:
                stack.append((arc.target, k, i + 1, sofar + text[i],
                              arc.accept))
        # If K is already 0, can't do any more edits
        if k < 1:
            continue
        k -= 1

        arcs = graph.arc_dict(address)
        # Insertions
        stack.extend((arc.target, k, i, sofar + char, arc.accept)
                     for char, arc in iteritems(arcs))

        # Deletion, replacement, and transpo only work before the end
        if i >= len(text):
            continue
        char = text[i]

        # Deletion
        stack.append((address, k, i + 1, sofar, False))
        # Replacement
        for char2, arc in iteritems(arcs):
            if char2 != char:
                stack.append((arc.target, k, i + 1, sofar + char2, arc.accept))
        # Transposition
        if i < len(text) - 1:
            char2 = text[i + 1]
            if char != char2 and char2 in arcs:
                # Find arc from next char to this char
                target = arcs[char2].target
                if target:
                    arc = graph.find_arc(target, char)
                    if arc:
                        stack.append((arc.target, k, i + 2,
                                      sofar + char2 + char, arc.accept))


# Utility functions

def dump_graph(graph, address=None, tab=0, out=None):
    if address is None:
        address = graph._root
    if out is None:
        out = sys.stdout

    here = "%06d" % address
    for i, arc in enumerate(graph.list_arcs(address)):
        if i == 0:
            out.write(here)
        else:
            out.write(" " * 6)
        out.write("  " * tab)
        out.write("%r %r %s %r\n"
                  % (arc.label, arc.target, arc.accept, arc.value))
        if arc.target is not None:
            dump_graph(graph, arc.target, tab + 1, out=out)



from __future__ import with_statement
import random, time, threading

import pytest

from whoosh import analysis, fields, query, writing
from whoosh.compat import b, u, xrange, text_type
from whoosh.filedb.filestore import RamStorage
from whoosh.util.testing import TempIndex


def test_no_stored():
    schema = fields.Schema(id=fields.ID, text=fields.TEXT)
    with TempIndex(schema, "nostored") as ix:
        domain = (u("alfa"), u("bravo"), u("charlie"), u("delta"), u("echo"),
                  u("foxtrot"), u("golf"), u("hotel"), u("india"))

        w = ix.writer()
        for i in xrange(20):
            w.add_document(id=text_type(i),
                           text=u(" ").join(random.sample(domain, 5)))
        w.commit()

        with ix.reader() as r:
            assert sorted([int(id) for id in r.lexicon("id")]) == list(range(20))


def test_asyncwriter():
    schema = fields.Schema(id=fields.ID(stored=True), text=fields.TEXT)
    with TempIndex(schema, "asyncwriter") as ix:
        domain = (u("alfa"), u("bravo"), u("charlie"), u("delta"), u("echo"),
                  u("foxtrot"), u("golf"), u("hotel"), u("india"))

        writers = []
        # Simulate doing 20 (near-)simultaneous commits. If we weren't using
        # AsyncWriter, at least some of these would fail because the first
        # writer wouldn't be finished yet.
        for i in xrange(20):
            w = writing.AsyncWriter(ix)
            writers.append(w)
            w.add_document(id=text_type(i),
                           text=u(" ").join(random.sample(domain, 5)))
            w.commit()

        # Wait for all writers to finish before checking the results
        for w in writers:
            if w.running:
                w.join()

        # Check whether all documents made it into the index.
        with ix.reader() as r:
            assert sorted([int(id) for id in r.lexicon("id")]) == list(range(20))


def test_asyncwriter_no_stored():
    schema = fields.Schema(id=fields.ID, text=fields.TEXT)
    with TempIndex(schema, "asyncnostored") as ix:
        domain = (u("alfa"), u("bravo"), u("charlie"), u("delta"), u("echo"),
                  u("foxtrot"), u("golf"), u("hotel"), u("india"))

        writers = []
        # Simulate doing 20 (near-)simultaneous commits. If we weren't using
        # AsyncWriter, at least some of these would fail because the first
        # writer wouldn't be finished yet.
        for i in xrange(20):
            w = writing.AsyncWriter(ix)
            writers.append(w)
            w.add_document(id=text_type(i),
                           text=u(" ").join(random.sample(domain, 5)))
            w.commit()

        # Wait for all writers to finish before checking the results
        for w in writers:
            if w.running:
                w.join()

        # Check whether all documents made it into the index.
        with ix.reader() as r:
            assert sorted([int(id) for id in r.lexicon("id")]) == list(range(20))


def test_updates():
    schema = fields.Schema(id=fields.ID(unique=True, stored=True))
    ix = RamStorage().create_index(schema)
    for _ in xrange(10):
        with ix.writer() as w:
            w.update_document(id=u("a"))
    assert ix.doc_count() == 1


def test_buffered():
    schema = fields.Schema(id=fields.ID, text=fields.TEXT)
    with TempIndex(schema, "buffered") as ix:
        domain = u("alfa bravo charlie delta echo foxtrot golf hotel india")
        domain = domain.split()

        w = writing.BufferedWriter(ix, period=None, limit=10,
                                   commitargs={"merge": False})
        for i in xrange(20):
            w.add_document(id=text_type(i),
                           text=u(" ").join(random.sample(domain, 5)))
        time.sleep(0.1)
        w.close()

        assert len(ix._segments()) == 2


def test_buffered_search():
    schema = fields.Schema(id=fields.STORED, text=fields.TEXT)
    with TempIndex(schema, "bufferedsearch") as ix:
        w = writing.BufferedWriter(ix, period=None, limit=5)
        w.add_document(id=1, text=u("alfa bravo charlie"))
        w.add_document(id=2, text=u("bravo tango delta"))
        w.add_document(id=3, text=u("tango delta echo"))
        w.add_document(id=4, text=u("charlie delta echo"))

        with w.searcher() as s:
            r = s.search(query.Term("text", u("tango")))
            assert sorted([d["id"] for d in r]) == [2, 3]

        w.add_document(id=5, text=u("foxtrot golf hotel"))
        w.add_document(id=6, text=u("india tango juliet"))
        w.add_document(id=7, text=u("tango kilo lima"))
        w.add_document(id=8, text=u("mike november echo"))

        with w.searcher() as s:
            r = s.search(query.Term("text", u("tango")))
            assert sorted([d["id"] for d in r]) == [2, 3, 6, 7]

        w.close()


def test_buffered_update():
    schema = fields.Schema(id=fields.ID(stored=True, unique=True),
                           payload=fields.STORED)
    with TempIndex(schema, "bufferedupdate") as ix:
        w = writing.BufferedWriter(ix, period=None, limit=5)
        for i in xrange(10):
            for char in u("abc"):
                fs = dict(id=char, payload=text_type(i) + char)
                w.update_document(**fs)

        with w.reader() as r:
            sfs = [sf for _, sf in r.iter_docs()]
            sfs = sorted(sfs, key=lambda x: x["id"])
            assert sfs == [{'id': u('a'), 'payload': u('9a')},
                           {'id': u('b'), 'payload': u('9b')},
                           {'id': u('c'), 'payload': u('9c')}]
            assert r.doc_count() == 3

        w.close()


def test_buffered_threads():
    domain = u("alfa bravo charlie delta").split()
    schema = fields.Schema(name=fields.ID(unique=True, stored=True))
    with TempIndex(schema, "buffthreads") as ix:
        class SimWriter(threading.Thread):
            def run(self):
                for _ in xrange(5):
                    w.update_document(name=random.choice(domain))
                    time.sleep(random.uniform(0.01, 0.1))

        w = writing.BufferedWriter(ix, limit=10)
        threads = [SimWriter() for _ in xrange(5)]
        for thread in threads:
            thread.start()
        for thread in threads:
            thread.join()
        w.close()

        with ix.reader() as r:
            assert r.doc_count() == 4
            assert sorted([d["name"] for d in r.all_stored_fields()]) == domain


def test_fractional_weights():
    ana = analysis.RegexTokenizer(r"\S+") | analysis.DelimitedAttributeFilter()

    # With Positions format
    schema = fields.Schema(f=fields.TEXT(analyzer=ana))
    ix = RamStorage().create_index(schema)
    w = ix.writer()
    w.add_document(f=u("alfa^0.5 bravo^1.5 charlie^2.0 delta^1.5"))
    w.commit()

    with ix.searcher() as s:
        wts = []
        for word in s.lexicon("f"):
            p = s.postings("f", word)
            wts.append(p.weight())
        assert wts == [0.5, 1.5, 2.0, 1.5]

    # Try again with Frequency format
    schema = fields.Schema(f=fields.TEXT(analyzer=ana, phrase=False))
    ix = RamStorage().create_index(schema)
    w = ix.writer()
    w.add_document(f=u("alfa^0.5 bravo^1.5 charlie^2.0 delta^1.5"))
    w.commit()

    with ix.searcher() as s:
        wts = []
        for word in s.lexicon("f"):
            p = s.postings("f", word)
            wts.append(p.weight())
        assert wts == [0.5, 1.5, 2.0, 1.5]


def test_cancel_delete():
    schema = fields.Schema(id=fields.ID(stored=True))
    # Single segment
    with TempIndex(schema, "canceldelete1") as ix:
        w = ix.writer()
        for char in u("ABCD"):
            w.add_document(id=char)
        w.commit()

        with ix.reader() as r:
            assert not r.has_deletions()

        w = ix.writer()
        w.delete_document(2)
        w.delete_document(3)
        w.cancel()

        with ix.reader() as r:
            assert not r.has_deletions()
            assert not r.is_deleted(2)
            assert not r.is_deleted(3)

    # Multiple segments
    with TempIndex(schema, "canceldelete2") as ix:
        for char in u("ABCD"):
            w = ix.writer()
            w.add_document(id=char)
            w.commit(merge=False)

        with ix.reader() as r:
            assert not r.has_deletions()

        w = ix.writer()
        w.delete_document(2)
        w.delete_document(3)
        w.cancel()

        with ix.reader() as r:
            assert not r.has_deletions()
            assert not r.is_deleted(2)
            assert not r.is_deleted(3)


def test_delete_nonexistant():
    from whoosh.writing import IndexingError

    schema = fields.Schema(id=fields.ID(stored=True))
    # Single segment
    with TempIndex(schema, "deletenon1") as ix:
        w = ix.writer()
        for char in u("ABC"):
            w.add_document(id=char)
        w.commit()

        try:
            w = ix.writer()
            with pytest.raises(IndexingError):
                w.delete_document(5)
        finally:
            w.cancel()

    # Multiple segments
    with TempIndex(schema, "deletenon1") as ix:
        for char in u("ABC"):
            w = ix.writer()
            w.add_document(id=char)
            w.commit(merge=False)

        try:
            w = ix.writer()
            with pytest.raises(IndexingError):
                w.delete_document(5)
        finally:
            w.cancel()


def test_add_field():
    schema = fields.Schema(a=fields.TEXT)
    with TempIndex(schema, "addfield") as ix:
        with ix.writer() as w:
            w.add_document(a=u("alfa bravo charlie"))
        with ix.writer() as w:
            w.add_field("b", fields.ID(stored=True))
            w.add_field("c*", fields.ID(stored=True), glob=True)
            w.add_document(a=u("delta echo foxtrot"), b=u("india"), cat=u("juliet"))

        with ix.searcher() as s:
            fs = s.document(b=u("india"))
            assert fs == {"b": "india", "cat": "juliet"}


def test_add_reader():
    schema = fields.Schema(i=fields.ID(stored=True, unique=True),
                           a=fields.TEXT(stored=True, spelling=True),
                           b=fields.TEXT(vector=True))
    with TempIndex(schema, "addreader") as ix:
        with ix.writer() as w:
            w.add_document(i=u("0"), a=u("alfa bravo charlie delta"),
                           b=u("able baker coxwell dog"))
            w.add_document(i=u("1"), a=u("bravo charlie delta echo"),
                           b=u("elf fabio gong hiker"))
            w.add_document(i=u("2"), a=u("charlie delta echo foxtrot"),
                           b=u("india joker king loopy"))
            w.add_document(i=u("3"), a=u("delta echo foxtrot golf"),
                           b=u("mister noogie oompah pancake"))

        with ix.writer() as w:
            w.delete_by_term("i", "1")
            w.delete_by_term("i", "3")

        with ix.writer() as w:
            w.add_document(i=u("4"), a=u("hotel india juliet kilo"),
                           b=u("quick rhubarb soggy trap"))
            w.add_document(i=u("5"), a=u("india juliet kilo lima"),
                           b=u("umber violet weird xray"))

        with ix.reader() as r:
            assert r.doc_count_all() == 4

            sfs = list(r.all_stored_fields())
            assert sfs == [{"i": u("4"),
                            "a": u("hotel india juliet kilo")},
                           {"i": u("5"), "a":
                            u("india juliet kilo lima")},
                           {"i": u("0"), "a":
                            u("alfa bravo charlie delta")},
                           {"i": u("2"), "a":
                            u("charlie delta echo foxtrot")},
                           ]

            assert " ".join(r.field_terms("a")) == "alfa bravo charlie delta echo foxtrot hotel india juliet kilo lima"

            vs = []
            for docnum in r.all_doc_ids():
                v = r.vector(docnum, "b")
                vs.append(list(v.all_ids()))
            assert vs == [["quick", "rhubarb", "soggy", "trap"],
                          ["umber", "violet", "weird", "xray"],
                          ["able", "baker", "coxwell", "dog"],
                          ["india", "joker", "king", "loopy"]
                          ]

            gr = r.word_graph("a")
            assert list(gr.flatten_strings()) == ["alfa", "bravo", "charlie",
                                                  "delta", "echo", "foxtrot",
                                                  "hotel", "india", "juliet",
                                                  "kilo", "lima"]


def test_add_reader_spelling():
    # Test whether add_spell_word() items get copied over in a merge

    # Because b is stemming and spelled, it will use add_spell_word()
    ana = analysis.StemmingAnalyzer()
    schema = fields.Schema(a=fields.TEXT(spelling=True),
                           b=fields.TEXT(analyzer=ana, spelling=True))
    with TempIndex(schema, "addreadersp") as ix:
        with ix.writer() as w:
            w.add_document(a=u("rendering modeling compositing enabling"),
                           b=u("rendering modeling compositing enabling"))
            w.add_document(a=u("flying rolling tying quitting polling"),
                           b=u("flying rolling tying quitting polling"))

        with ix.writer() as w:
            w.add_document(a=u("writing eyeing ripping timing yelling"),
                           b=u("writing eyeing ripping timing yelling"))
            w.add_document(a=u("undoing indicating opening pressing"),
                           b=u("undoing indicating opening pressing"))

        with ix.searcher() as s:
            gr = s.reader().word_graph("a")
            assert " ".join(gr.flatten_strings()) == ("compositing enabling eyeing flying indicating "
                                                      "modeling opening polling pressing quitting "
                                                      "rendering ripping rolling timing tying undoing "
                                                      "writing yelling")

            gr = s.reader().word_graph("b")
            assert " ".join(gr.flatten_strings()) == ("compositing enabling eyeing flying indicating "
                                                      "modeling opening polling pressing quitting "
                                                      "rendering ripping rolling timing tying undoing "
                                                      "writing yelling")

def test_clear():
    schema = fields.Schema(a=fields.KEYWORD)
    ix = RamStorage().create_index(schema)

    # Add some segments
    with ix.writer() as w:
        w.add_document(a=u("one two three"))
        w.merge = False
    with ix.writer() as w:
        w.add_document(a=u("two three four"))
        w.merge = False
    with ix.writer() as w:
        w.add_document(a=u("three four five"))
        w.merge = False

    # Clear
    with ix.writer() as w:
        w.add_document(a=u("foo bar baz"))
        w.mergetype = writing.CLEAR

    with ix.searcher() as s:
        assert s.doc_count_all() == 1
        assert list(s.reader().lexicon("a")) == [b("bar"), b("baz"), b("foo")]


def test_spellable_list():
    # Make sure a spellable field works with a list of pre-analyzed tokens

    ana = analysis.StemmingAnalyzer()
    schema = fields.Schema(Location=fields.STORED,Lang=fields.STORED,
                           Title=fields.TEXT(spelling=True, analyzer=ana))
    ix = RamStorage().create_index(schema)

    doc = {'Location': '1000/123', 'Lang': 'E',
           'Title': ['Introduction', 'Numerical', 'Analysis']}

    with ix.writer() as w:
        w.add_document(**doc)


def test_zero_procs():
    schema = fields.Schema(text=fields.TEXT)
    ix = RamStorage().create_index(schema)
    with ix.writer(procs=0) as w:
        assert isinstance(w, writing.IndexWriter)

    with ix.writer(procs=1) as w:
        assert isinstance(w, writing.IndexWriter)



from __future__ import with_statement

from whoosh import fields, formats
from whoosh.compat import u
from whoosh.filedb.filestore import RamStorage
from whoosh.util.testing import TempIndex


def test_single_term():
    schema = fields.Schema(text=fields.TEXT(vector=True))
    ix = RamStorage().create_index(schema)
    with ix.writer() as w:
        w.add_document(text=u("TEST TEST TEST"))
    with ix.searcher() as s:
        v = s.vector(0, "text")
        assert v.is_active()


def test_vector_reading():
    schema = fields.Schema(title=fields.TEXT,
                           content=fields.TEXT(vector=formats.Frequency()))

    with TempIndex(schema, "vectorreading") as ix:
        writer = ix.writer()
        writer.add_document(title=u("one"),
                            content=u("This is the story of the black " +
                                      "hole story"))
        writer.commit()

        with ix.reader() as r:
            assert list(r.vector_as("frequency", 0, "content")) == [(u('black'), 1), (u('hole'), 1), (u('story'), 2)]


def test_vector_merge():
    schema = fields.Schema(title=fields.TEXT,
                           content=fields.TEXT(vector=formats.Frequency()))

    with TempIndex(schema, "vectormerge") as ix:
        writer = ix.writer()
        writer.add_document(title=u("one"),
                            content=u("This is the story of the black hole " +
                                      "story"))
        writer.commit()

        writer = ix.writer()
        writer.add_document(title=u("two"),
                            content=u("You can read along in your book"))
        writer.commit()

        with ix.searcher() as s:
            r = s.reader()

            docnum = s.document_number(title=u("one"))
            vec = list(r.vector_as("frequency", docnum, "content"))
            assert vec == [(u('black'), 1), (u('hole'), 1), (u('story'), 2)]

            docnum = s.document_number(title=u("two"))

            vec = list(r.vector_as("frequency", docnum, "content"))
            assert vec == [(u('along'), 1), (u('book'), 1), (u('read'), 1)]


def test_vector_unicode():
    cf = fields.TEXT(vector=True)
    schema = fields.Schema(content=cf)
    ix = RamStorage().create_index(schema)

    writer = ix.writer()
    writer.add_document(content=u("\u13a0\u13a1\u13a2 \u13a3\u13a4\u13a5"))
    writer.add_document(content=u("\u13a6\u13a7\u13a8 \u13a9\u13aa\u13ab"))
    writer.commit()

    writer = ix.writer()
    writer.add_document(content=u("\u13ac\u13ad\u13ae \u13af\u13b0\u13b1"))
    writer.add_document(content=u("\u13b2\u13b3\u13b4 \u13b5\u13b6\u13b7"))
    writer.commit()

    with ix.reader() as r:
        vec = list(r.vector_as("frequency", 0, "content"))
        assert vec == [(u('\u13ac\u13ad\u13ae'), 1), (u('\u13af\u13b0\u13b1'), 1)]


def test_add_vectored_field():
    schema = fields.Schema(id=fields.ID(stored=True), f1=fields.TEXT)
    ix = RamStorage().create_index(schema)
    with ix.writer() as w:
        w.add_document(id=u("a"), f1=u("Testing one two three"))

    with ix.writer() as w:
        w.add_field("f2", fields.TEXT(vector=True))
        w.add_document(id=u("b"), f2=u("Frosting four five six"))

    with ix.searcher() as s:
        docnum1 = s.document_number(id="a")
        assert not s.has_vector(docnum1, "f1")

        docnum2 = s.document_number(id="b")
        assert not s.has_vector(docnum2, "f1")
        assert s.has_vector(docnum2, "f2")
